
"""Two standard sorting functions.   To test these, open a command window and
   start Python interactively:

   python -i sort.p
   >>> selectionSort([4,3,2,1])
   [4, 3, 2, 1]
   >>> a = [2, 3, 2, 5]
   >>> b = selectionSort(a)
   >>> print b
   [2, 2, 3, 5]
   >>> print a
   [2, 3, 2, 5]
   >>>
   >>> mergeSort(a)
   [2, 2, 3, 5]
   >>> 
"""
     

def selectionSort(r) :
    """selectionSort sorts the elements of  r  by means of selection sorting.

       param:  r -- a list (array) of ints
       returns: ans, a new list that has exactly  r's  elements, sorted by <=
    """
    ans = []   # names the sorted listed, which grows one elements at a time
    while r != [] :
        # invariant: ans  is sorted, and all elements in  r  are >=  those in  ans
        # first, find least element remaining in r :
        leastindex = 0
        index = 1
        while index < len(r) :
            # invariant:  r[leastindex] is smallest in  r[0]..r[leastindex]-1
            if r[index] < r[leastindex] :
                leastindex = index
            index = index + 1
        ans.append(r[leastindex])   # move least element to ans
        r = r[:leastindex] + r[leastindex+1:] # remove least element from r
        
    return ans


def mergeSort(r) :
    """mergeSort sorts the elements of  r  by means of merge sorting.

       param:  r -- a list (array) of ints
       returns: ans, a new list that has exactly  r's  elements, sorted by <=
    """

    def merge(a, b) :  # a private function
        """merge combines the elements of sorted lists  a  and  b  into
           a new list,  c,  that is also sorted
        """
        c = []
        while len(a) > 0 and len(b) > 0 :
            # invariant: all elements in  c  are  <=  those remaining in  a  and  b
            if a[0] < b[0] :
                c.append(a[0])
                a = a[1:]
            else :
                c.append(b[0])
                b = b[1:]
        c = c + a + b   # add to end of  ans  any leftover elements
        return c
    # end of private function

    if len(r) <= 1 :
        ans = r
    else :
        midpoint = len(r) / 2    # integer division drops the fraction
        sortedleft = mergeSort(r[:midpoint])
        sortedright = mergeSort(r[midpoint:])
        ans = merge(sortedleft, sortedright)
    return ans

